/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2012 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.lookandfeel.ui.quaqua;

import java.awt.Color;
import java.awt.Graphics;
import java.lang.reflect.Method;
import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import ch.randelshofer.quaqua.QuaquaTreeUI;

/*******************************************************************************
 *
 * Patched version of <code>QuaquaTreeUI</code> which honors the selection 
 * colors of a <code>DefaultTreeCellRenderer</code>.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class BMQuaquaTreeUI extends QuaquaTreeUI
  {
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static BMQuaquaTreeUI createUI (JComponent component) 
      {
        return new BMQuaquaTreeUI();
      }    
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public void paint (final Graphics g, final JComponent c)  
      {
        try 
          {
            final TreeCellRenderer cellRenderer = ((JTree)c).getCellRenderer();
            final Class rendererClass = cellRenderer.getClass();
            final Method getTextSelectionColorMethod = rendererClass.getMethod("getTextSelectionColor");
            final Method getBackgroundSelectionColorMethod = rendererClass.getMethod("getBackgroundSelectionColor");
            final Color selectionBackgroundSave = UIManager.getColor("Tree.selectionBackground");
            final Color selectionForegroundSave = UIManager.getColor("Tree.selectionForeground");        
            UIManager.put("Tree.selectionBackground", getBackgroundSelectionColorMethod.invoke(cellRenderer));
            UIManager.put("Tree.selectionForeground", getTextSelectionColorMethod.invoke(cellRenderer));
            super.paint(g, c);
            UIManager.put("Tree.selectionBackground", selectionBackgroundSave);
            UIManager.put("Tree.selectionForeground", selectionForegroundSave);
          } 
        catch (NoSuchMethodException e) 
          {
            super.paint(g, c);
          }
        catch (Exception e) 
          {
            e.printStackTrace();
            super.paint(g, c);
          }
      }  
  }
